@@ -8,6 +8,8 @@
 ########################################################################
 
 use Test::Spec;
+use FindBin qw($Bin);
+BEGIN { require "$Bin/test_helper.pl" };
 
 describe "Test::Spec" => sub {
   it "turns on perl warnings in test file" => sub {
